<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\Cegid\Helper\Cegid\Order;
use Magento\Rma\Api\Data\RmaInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use RuntimeException;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Create;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\OmniChannel;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentIdentifier;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentReference;
use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfUserDefinedDate;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\UserDefinedDate;
use Magento\Framework\App\ResourceConnection;

/**
 * Class OrderReturnFromRma
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class OrderReturnFromRma extends Order {

    public function createReturn($rma_object){

        $already_sent = $rma_object->getData('sent_to_ext');
        if ($already_sent){
            return true;
        }

        $items = $rma_object->getItems();
        if (count($items) === 0){
            $this->writeLog('createReturnFromRma',"none","zero items","rma increment: {$rma_object->getIncrementId()} | order increment:  {$rma_object->getOrderIncrementId()} ");
            return true;
        }

        $this->is_return = true;
        $order = $this->current_order = $rma_object->getOrder();
        $this->orderCegidInit($order);
        $sku_qty_array_to_return = array();

        foreach ($items as $item) {
            $sku_qty_array_to_return[$item->getProductSku()] = $item->getQtyRequested();
        }
        $a=1;

        if (empty($sku_qty_array_to_return)){
            $this->writeLog('createReturnFromRma',"none","zero products","rma increment: {$rma_object->getIncrementId()} | order increment:  {$rma_object->getOrderIncrementId()} ");
            return true;
        }

        // The current source code in this case depends on the products that we are trying to return
        // Since with a multistock order we split the calls to CEGID (ex. ORDER1-SOURCE1, ORDER1-SOURCE2...)
        // Now we need to backtrace which source was connected to this product
        $this->current_source_code = $this->findOriginalSourceBySku($rma_object);

        $skip = $this->shouldSkipExecution(get_class($this),__FUNCTION__,array('source_code' => $this->current_source_code, 'processing_object' => $rma_object));

        if (!$this->current_source_code || $skip){
            // No products inside this rma or source_code is missing
            return false;
        }

        // Get base header
        $header = $this->getHeader($order);
        //change internal and external reference according to Marco Zanotto
        $header->setInternalReference($header->getInternalReference()."_R");
        $header->setExternalReference($header->getExternalReference()."_R");

        // Change some parameters because this is a return
        $header->setLinesUnmodifiable(false);
        $omnichannel = $this->getReturnOmniChannel($order);
        $header->setOmniChannel($omnichannel);
        $header->setType('ReturnNotice');

        // Set return as already accepted
        $header->setUserDefinedDates($this->getUserDefinedDates($rma_object));

        $lines = $this->getReturnLines($rma_object,$sku_qty_array_to_return);

        /** @var Create_Request $request */
        $request = $this->createRequestFactory->create([
            'header' => $header
        ]);

        $request->setLines($lines);

        /** @var \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create $parameters */
        $parameters = $this->createFactory->create();
        $parameters->setCreateRequest($request);
        $parameters->setClientContext($this->getRetailContext());
        // Execute
        $createService = $this->createService->create();
        $createService->Create($parameters);
        // Save data
        $request = $createService->getLastRequest();
        $response = $createService->getLastResponse();
        $error = array_values($createService->getLastError());
        $result = $createService->getResult();

        $this->writeLog('createReturnFromRma',$request,$response,$order);

        if(!$result && $error && count($error)){
            return $error[0]->getMessage();
        }
        $rma_object->setData('sent_to_ext',1)->save();
        return true;

        //throw new RuntimeException(implode("\n", $array_of_error_messages));
    }

    public function getUserDefinedDates($rma_object){
        $userDefinedDate = new UserDefinedDate();
        $userDefinedDate->setId('1');
        $userDefinedDate->setValue(($this->getDateObjectTimezonedForCegid($rma_object->getDateRequested()))->format('Y-m-d'));

        $userDefinedDates = new ArrayOfUserDefinedDate();
        $userDefinedDates->addToUserDefinedDate($userDefinedDate);

        return $userDefinedDates;
    }

    public function getReturnOmniChannel(OrderInterface $order){
        $omnichannel = new OmniChannel();
        $omnichannel->setOriginalDocument($this->getReturnOmniChannelOriginalDocument($order));
        $omnichannel->setBillingStatus('NotPerformed');
        $omnichannel->setFollowUpStatus('WaitingForReturn');
        // TODO: payment method?
        //$omnichannel->setPaymentMethodId($this->getCegidPaymentMethodId($order));
        $omnichannel->setPaymentMethodId("RMB");
        $omnichannel->setReturnStatus('ReturnPlanned');
        $omnichannel->setReturnType('Return');
        $omnichannel->setShippingStatus('Pending');
        return $omnichannel;
    }

    public function getReturnOmniChannelOriginalDocument(OrderInterface $order){
        $reference = new SaleDocumentReference();
        $reference->setCustomerId($this->getCustomerCegidFromOrder($order));
        $reference->setInternalReference($this->getOrderReference($order,$this->current_source_code));
        $reference->setType(SaleDocumentType::VALUE_CUSTOMER_ORDER);
        $originalDocument = new SaleDocumentIdentifier();
        $originalDocument->setReference($reference);
        return $originalDocument;
    }

    /**
     * @param RmaInterface $rma_object
     * @param $sku_qty_array_to_return
     * @return \FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_Line
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getReturnLines($rma_object,$sku_qty_array_to_return){
        $lines = $this->arrayOfCreateLineFactory->create();
        $origin = $this->getConfigSaleDocumentOrigin();
        $order = $rma_object->getOrder();
        foreach ($rma_object->getItems() as $rma_item) {

            // Rma object items does not have the data CEGID wants, getting price data on the order items
            /** @var OrderItemInterface|null $item */
            $item = null;
            foreach ($order->getItems() as $order_item) {
                if($order_item->getId() == $rma_item->getOrderItemId()){
                    $item = $order_item;
                    break;
                } else {
                    continue;
                }
            }

            if(!$item || $item->getPrice() == 0){
                continue;
            }

            // I need to split the grand total for each source, and everything must be in EUR
            // This MUST BE with VAT
            $single_unit_row_total = $item->getBaseRowTotalInclTax() / $rma_item->getQtyRequested();
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $single_unit_row_total *= $order->getBaseToGlobalRate();
            }

            $line = $this->createLineFactory->create();

            $loaded_product = $this->productRepositoryInterface->get($item->getSku());
            $barcode = $loaded_product->getBarcode();

            $qty_to_take = $sku_qty_array_to_return[$item->getSku()];

            $this_order_row_total = $single_unit_row_total * $qty_to_take;
            $this->current_order_to_cegid_grandtotal += $this_order_row_total;

            $identifier = $this->itemIdentifierFactory->create(['reference' => $barcode]);
            $line->setItemIdentifier($identifier);
            $line->setOrigin($origin);
            $line->setQuantity($qty_to_take); //Not $item->getQtyOrdered()

            // Importo Iva a default articolo. In questo caso non va passata l’intera sezione Taxes
            //$line->setTaxes($this->getTaxes($item)); // This must be before the unit price

            // Importo unitario (UnitPrice) e Importo Netto Sconti (Net Unit Price) IVA ESCLUSA.
            $unit_price = $item->getBaseRowTotal() / $rma_item->getQtyRequested();
            // Importo unitario (UnitPrice) e Importo Netto Sconti (Net Unit Price) IVA ESCLUSA.
            $net_unit_price = ($item->getBaseRowTotal() - ($item->getBaseDiscountAmount() - (float)$item->getBaseDiscountTaxCompensationAmount() )) / $rma_item->getQtyRequested();
            
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $net_unit_price *= $order->getBaseToGlobalRate();
                $unit_price *= $order->getBaseToGlobalRate();
            }
            
            // Include discount
            $line->setNetUnitPrice($net_unit_price);
            // prezzo IVA ESCLUSA SENZA SCONTI di una unità
            $line->setUnitPrice($unit_price);
            //nell'rma non vogliono l'omnichannel di linea
            // $omniChannelLine = $this->getCegidReturnOmniChannelLine($this->current_source_code);
            //$line->setOmniChannel($omniChannelLine);
            $lines->addToCreate_Line($line);
        }
        return $lines;
    }

    public function getCegidReturnOmniChannelLine($warehouse_code)
    {
        $third_party = $this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($warehouse_code);
        $omniChannelLine = $this->omniChannelLineFactory->create();
        $omniChannelLine->setWarehouseId($third_party);
        return $omniChannelLine;
    }

    public function findOriginalSourceBySku($rma_object){
        $array_of_source_codes = array();

        $array_of_order_items_id = array();
        $items = $rma_object->getItems();
        foreach ($items as $item) {
            $array_of_order_items_id[$item->getProductSku()] = $item->getOrderItemId();
        }

        foreach ($array_of_order_items_id as $sku => $order_item_id) {
            $sql = "SELECT inventory_source_id 
                    FROM inventory_fb_orderitem_source IFOS 
                    WHERE IFOS.order_item_id = (
                        SELECT SOI.item_id
                        FROM sales_order_item SOI
                        WHERE SOI.parent_item_id = {$order_item_id}
                        AND sku = '{$sku}'
                        AND product_type = 'simple' LIMIT 1
                    )";
            $result = $this->getConnection()->fetchOne($sql);

            if ($result === false){
                // If I can't find any source code from inventory_fb_orderitem_source then fallback to default
                $main_source = $this->extInvHelperSources->getSource($this->stock_id,$this->sourceSlotsSourcesData->getMainSourceCode());
                $array_of_source_codes[] = $main_source->getSourceCode();
            } else {
                $array_of_source_codes[] = $result;
            }

        }

        $array_of_source_codes = array_values(array_filter(array_unique($array_of_source_codes)));

        return $array_of_source_codes[0];
    }

    public function checkOrderRmaAcceptedStatus($rma_object){
        $order = $rma_object->getOrder();
        $this->orderCegidInit($order);

        $source_code = $this->findOriginalSourceBySku($rma_object);
        $this->current_source_code = $source_code;

        $skip = $this->shouldSkipExecution(get_class($this),__FUNCTION__,array('source_code' => $source_code, 'processing_object' => $order, 'processing_object_id' => $order->getIncrementId()));
        $result = true;
        if(!$skip){
            $result = $this->getByReference($order,true);
        }

        /*
         *
         if (!$result){
            return false;
        }
        */
        return $result;
    }


    public function checkRmaReceipt($rma_object){

        $order = $rma_object->getOrder();
        $this->orderCegidInit($order);
        $source_code = $this->findOriginalSourceBySku($rma_object);
        $this->current_source_code = $source_code;

        $result = $this->getReceipt($order);

        return $result;


    }






}