<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\Cegid\Helper\Cegid\Order;
use RuntimeException;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\ServiceType\Create as CreateService;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequest;
use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\Create;
use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestHeader;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestHeaderRecipient;
use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestHeaderSender;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\EnumType\ItemsDeliveryDocumentTypeCreat;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestLineCollection;
use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestLine;
use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\CreateRequestLineItemIdentifier;

use FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\RetailContext;

use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfUserDefinedDate;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\UserDefinedDate;
use Magento\Framework\App\ResourceConnection;

/**
 * Class TransferItemsToWarehouse
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class TransferItemsToWarehouse extends Order {

    public $source_warehouse;
    public $destination_warehouse;

    public function createTransfer($order){
        $this->orderCegidInit($order);
        $this->current_order = $order;

        if (!$this->isTransferAuthorized()){
            $this->writeLog('TransferItemsToWarehouse','createTransfer',"will skip this execution! not authorized!",$order->getIncrementId());
            return true;
        }

        if ($this->getItemsToTransfer() === false){
            $this->writeLog('TransferItemsToWarehouse','createTransfer',"will skip this execution! no items to transfer!",$order->getIncrementId());
            return true;
        }

        $header = $this->getHeaderData();
        $lines_collection = $this->getTransferLinesCollection();
        $request = new CreateRequest($header); //TODO
        $request->setLines($lines_collection);

        /** @var \FiloBlu\CegidY2\Api\TransferItemsDeliveryManagementService\StructType\Create $parameters */
        $parameters = $this->createTransferFactory->create();

        $parameters->setRequest($request);
        $parameters->setContext($this->getTransferRetailContext());

        // Execute
        $createService = $this->createTransferService->create();
        $createService->Create($parameters);

        // Save data
        $request = $createService->getLastRequest();
        $response = $createService->getLastResponse();
        $error = array_values($createService->getLastError());
        $result = $createService->getResult();

        $this->writeLog('createTransfer',$request,$response,$order);

        if(!$result && $error && count($error)){
            return $error[0]->getMessage();
        }
        return true;
    }

    public function isTransferAuthorized($order = null){
        if(!$order){
            $order = $this->current_order;
        }
        if(!$order){
            return false;
        }
        $transfer_enabled = $this->isTransferEnabled($order->getStore()->getId());
        return $transfer_enabled;
    }

    /**
     * @return \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Header|CreateRequestHeader
     */
    public function getHeaderData(){
        $header = new CreateRequestHeader();
        $header->setDate(substr($this->current_order->getCreatedAt(), 0, 10));
        $header->setDocumentTypeToCreate(ItemsDeliveryDocumentTypeCreat::VALUE_SENT_TRANSFER);
        $header->setInternalReference($this->getOrderReference($this->current_order));
        $header->setRecipient($this->getRecipient());
        $header->setSender($this->getSender());
        return $header;
    }

    public function getRecipient(){
        $storeId = $this->getConfigCegidStoreId();
        $warehouseId = $this->getConfigCegidReceiverWarehouseId();
        $recipient = new CreateRequestHeaderRecipient($storeId,$warehouseId);
        return $recipient;
    }

    public function getSender(){
        $storeId = $this->getConfigCegidStoreId();
        $warehouseId = $this->getConfigCegidSenderWarehouseId();
        $sender = new CreateRequestHeaderSender($storeId,$warehouseId);
        return $sender;
    }

    public function getTransferLinesCollection(){
        $lines = $this->getLinesData();
        $linesCollection = new CreateRequestLineCollection($lines);
        return $linesCollection;
    }

    public function getLinesData(){
        $data = array();
        $items = $this->getItemsToTransfer();
        if ($items === false){
            return $data;
        }

        foreach ($items as $sku => $values) {
            $barcode = $values['barcode'];
            $qty = $values['qty'];
            $line = new CreateRequestLine();
            $line->setItemIdentifier((new CreateRequestLineItemIdentifier())->setReference($barcode));
            $line->setQuantity($qty);
            $line->setSenderWarehouseId($this->getConfigCegidSenderWarehouseId());
            $data[] = $line;
        }

        return $data;
    }

    /**
     * @return RetailContext
     */
    public function getTransferRetailContext()
    {
        return $this->transferRetailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('transfer_items_delivery_management_service')
        ]);
    }

    public function getConfigCegidStoreId($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/cegid_store_id', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        if(!$return){
            return '000';
        }
        return (string)$return;
    }

    public function getConfigCegidSenderWarehouseId($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/cegid_warehouse_id_sender', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        if(!$return){
            return '000000';
        }
        return (string)$return;
    }

    public function getConfigCegidReceiverWarehouseId($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/cegid_warehouse_id_receiver', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        if(!$return){
            return '000000';
        }
        return (string)$return;
    }

    public function getTransferFrom($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/transfer_from', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        if(!$return){
            return $this->sourceSlotsSourcesData->getMainSourceCode();
        }
        return (string)$return;
    }

    public function getTransferTo($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/transfer_to', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        if(!$return){
            return $this->sourceSlotsSourcesData->getMainSourceCode();
        }
        return (string)$return;
    }

    public function isTransferEnabled($storeview_id = null){
        $return = $this->scopeConfig->getValue('filoblu_cegid_y2/transfer_items_delivery_management_service/transfer_enabled', \Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeview_id);
        return $return;
    }

    public function getItemsToTransfer(){
        $order_storeview_id = 0;
        if ($this->current_order){
            $order_storeview_id = $this->current_order->getStore()->getId();
        }

        $transfer_from = $this->getTransferFrom($order_storeview_id);

        if (!isset($this->order_source_items_quantities[$transfer_from])) {
            return false;
        }

        $items = $this->order_source_items_quantities[$transfer_from];
        if (count($items) == 0){
            return false;
        }
        return $items;
    }

}